//---CAPTEUR DE CO2----Novembre 2020 - Tony Vanpoucke (Edulab, Rennes 2)----------------------------//
//                                                                                                  //
//   Petit capteur de C02 avec affichage 7 segements et triple seuil d'alerte                       //
//   Bibliothèques pré-requises : TM1637Display (Avishay Orpaz, IL) et MHZ19 (Jonathan Dempsey, UK) //
//                                                                                                  //
//--------------------------------------------------------------------------------------------------//

// ----- PARAMETRES MODIFIABLES ----------------------------------//

int SeuilPPM1 = 900;        //1er Seuil d'alerte au CO2 (900 PPM : air bon à moyen)
int SeuilPPM2 = 1600;       //2nd Seuil d'alerte au CO2 (1600 PPM : air moyen à mauvais, ventillation conseillee)
int SeuilPPM3 = 2200;       //3e Seuil d'alerte au CO2 (2200 PPM : mauvais air, maux de tetes, malaise possible, ventillation indispensable)

int SensorInterval = 2000;  //Interval de capture du CO2

float luminosite = 2.5;   //luminosité des écrans
int loadTime = 10000;     //temps de "chauffe" du capteur CO2
int loadSpeed = 35;       //rapiditée de l'animation de chargement
int messageInterval = 4;  //fréquence des messages d'alerte si un seuil CO2 est passé (4 par défaut)

// ----- Variables du programme

int CO2PPM = 0;
int messageBox = 0;

// ----- Bibliothèques utilisées

#include <Arduino.h>
#include <SoftwareSerial.h>
#include <TM1637Display.h>
#include "MHZ19.h"

#define BAUDRATE 9600                       // fréquence de fonctionnement du capteur CO2 (ne pas changer)

// ----- Branchements arduino

// Branchelent capteur MHZ16B
#define RX_PIN 10
#define TX_PIN 11

// Branchelent ecran 7 segement
#define CLK 4
#define DIO 5

// Raccordement electronique a la Arduino
MHZ19 myMHZ19;
SoftwareSerial mySerial(RX_PIN, TX_PIN);    
TM1637Display displayScreen(CLK, DIO);
unsigned long getDataTimer = 0;

// ----- INITIALISATION ---------------------------------------//

void setup()
{
  // Demarre la communication serie avec la Arduino
  Serial.begin(9600);

  // Initialise la lecture du capteur C02
  mySerial.begin(BAUDRATE);
  myMHZ19.begin(mySerial);
  myMHZ19.setFilter(true, true);
  myMHZ19.autoCalibration(true);

  CO2PPM = myMHZ19.getCO2(true, true);                        // Capture de la donnée pour "chauffer" le capteur de CO2

  displayScreen.setBrightness(luminosite);                    // Applique la lumisosité à l'ecran 7 seg

  loadingAnimation();                                         // Lance une animation ecran le temps de "chauffer" la capteur CO2

}

// ----- PROGRAMME ---------------------------------------//

void loop()
{
  // Par intervals régulier capture la donnée et l'affiche :
  if (millis() - getDataTimer >= SensorInterval)
  {
    CO2PPM = myMHZ19.getCO2(true, true);    // Numérisation de la donnée de CO2
    
    Serial.println(CO2PPM);    // affichage du taux de CO2 sur le port série Arduino

    getDataTimer = millis();   // stocker les donnée d'interval

    screenDisplay();           // afficher la donnee sur l'ecran 7 segement (si un seuil est atteint, declanche animation)
  }
}


// --------------------------------------------------------//

void loadingAnimation()
{
  uint8_t loadA[] = { 0x30, 0x00, 0x00, 0x00 };                //animation de chargment 1/5
  uint8_t loadB[] = { 0x06, 0x30, 0x00, 0x00 };                //animation de chargment 2/5
  uint8_t loadC[] = { 0x00, 0x06, 0x30, 0x00 };                //animation de chargment 3/5
  uint8_t loadD[] = { 0x00, 0x00, 0x06, 0x30 };                //animation de chargment 4/5
  uint8_t loadE[] = { 0x00, 0x00, 0x00, 0x06 };                //animation de chargment 5/5

  //Affiche l'aniamtion en boucle
  for (int i = 0; i <= loadSpeed; i++) {
    displayScreen.setSegments(loadA);
    delay((loadTime / loadSpeed) / 5);
    displayScreen.setSegments(loadB);
    delay((loadTime / loadSpeed) / 5);
    displayScreen.setSegments(loadC);
    delay((loadTime / loadSpeed) / 5);
    displayScreen.setSegments(loadD);
    delay((loadTime / loadSpeed) / 5);
    displayScreen.setSegments(loadE);
    delay((loadTime / loadSpeed) / 5);
  }
}


void screenDisplay()
{
  // Si une des valeurs CO2 se trouve entre le seuil 1 et le seuil 2 :
  
  if ((SeuilPPM1 < CO2PPM) && (SeuilPPM2 > CO2PPM)) {                           
    if (messageBox == messageInterval) {
      uint8_t LowAlert[] = { 0x39, 0x5c, 0x5b, 0x08 }; //Affiche le message d'alerte (Low)
      displayScreen.setSegments(LowAlert);
      messageBox = 0;
    } else {
      displayScreen.showNumberDecEx(CO2PPM, false);    //Affiche les valeurs dans les écrans 7 segements
      messageBox++;
    }

  // Si une des valeurs CO2 se trouve entre le seuil 2 et le seuil 3 :  
  } else if ((SeuilPPM2 < CO2PPM) && (SeuilPPM3 > CO2PPM)) {  
  if (messageBox == messageInterval/messageInterval) {
      uint8_t MedAlert[] = { 0x39, 0x5c, 0x5b, 0x40 }; //Affiche le message d'alerte (Med)
      displayScreen.setSegments(MedAlert);
      messageBox = 0;
    } else {
      displayScreen.showNumberDecEx(CO2PPM, false);    //Affiche les valeurs dans les écrans 7 segements
      messageBox++;
    }

  // Si une des valeurs CO2 se trouve au dela du seuil 3 : 
  } else if (SeuilPPM3 < CO2PPM) { 
      if (messageBox == messageInterval/messageInterval) {
      uint8_t HiAlert[] = { 0x39, 0x5c, 0x5b, 0x01 }; //Affiche le message d'alerte (High)
      displayScreen.setSegments(HiAlert);
      messageBox = 0;
    } else {
      displayScreen.showNumberDecEx(CO2PPM, false);    //Affiche les valeurs dans les écrans 7 segements
      messageBox++;
    }

  // Si une des valeurs CO2 se trouve sous le seuil 1 (alors il n'y a pas l'alerte) : 
  } else {
    displayScreen.showNumberDecEx(CO2PPM, false);    //Sinon valeurs dans les écrans 7 segements (sans message d'alerte)
    messageBox = 0;
  }
}
